/******************************************************************************/
/* SAS macro _Count_  vs. 0.9.5 , 18 Nov 2008, by Jim Groeneveld, Netherlands */
/******************************************************************************/
/*                                                                            */
   %MACRO   _Count_   /* counts occurring value lists for (dataset) variables */
/* -------------------------------------------------------------------------- */
/* ---------- A l w a y s   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  SPSScode /* positional arg value: SPSS-like COUNT code between parens    R */
/*                                                                            */
/* ---------- R a r e l y   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  GlobIncr= _ArrayNr  /* name of unique global macro variable with incr.   O */
,  ArPrefix= _0        /* prefix of unique temporary SAS array for TO-list  O */
,  IndexVar= _0_       /* name of unique index variable for array elements  O */
,  Assign  = =         /* assignment character for counting variable        O */
,  NewCount= /         /* separator character between different counts      O */
,  LeftPar = %STR(%()  /* value list left, start char., default left paren  O */
,  RightPar= %STR(%))  /* value list right, end char., default right paren  O */
                       /* {only allowed to redefine both LeftPar, RightPar}   */
,  Space   = %STR( )   /* 1st element separator in variable and value lists O */
,  Comma   = %STR(,)   /* 2nd element separator in variable and value lists O */
,  Sysmis  = _SYSMIS_  /* keyword for all 28 missing values ._ . .a to .z   O */
,  To      = _TO_      /* list keyword for variable list (always also --)   O */
,  Thru    = _THRU_    /* range keyword for value range including limits    O */
,  Till    = _TILL_    /* range keyword for value range excluding limits    O */
,  Low     = _LO_      /* LOW  keyword for infinite negative value range    O */
,  High    = _HI_      /* HIGH keyword for infinite positive value range    O */
,  Period  = .         /* optional ending character of SPSS command         O */
,  Version = 0.9.5     /* forced version specification for version control  O */
/*                                                                            */
/* ----------- N e v e r   s p e c i f i e d   a r g u m e n t s ------------ */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  Debug   = 0         /* Debug level; for testing, debugging purposes only O */
,  ForMyUse=           /* argument for the program's use only, don't use it O */
)/ DES     =              'SPSS compatible COUNT command' /*Store*/            ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : _Count_.SAS / _Count_.095                                     *;
%*                                                                            *;
%* Purpose    : counts occurring value lists for variables, like SPSS COUNT   *;
%*                                                                            *;
%* Programmer : Jim Groeneveld                                                *;
%* Date       : 18 Nov 2008                                                   *;
%* Version    : 0.9.5                                                         *;
%*                                                                            *;
%* SAS version: 9.1 under Windows XP (not (entirely) 8.x compatible anymore)  *;
%*                                                                            *;
%* Validation : -                                                             *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Summary:                                                                   *;
%* ========                                                                   *;
%* The macro %_Count_ simulates the SPSS COUNT command in SAS. It counts      *;
%* the occurrence of specific values (value lists) in specified variables     *;
%* (variable lists) within records (SPSS cases). Results are stored in        *;
%* user specified variables. SAS doesnt have an equivalent statement and     *;
%* programming many sets of conditional counts requires quite a lot of data   *;
%* step code. The macro %_Count_ makes life easy by allowing a virtually      *;
%* identical syntax and functionality as the SPSS command. It even supports   *;
%* syntax and functionality beyond the SPSS COUNT. Its main features are:     *;
%* 1. Full support of the SPSS syntax as the first (positional) argument      *;
%*    value (between outer parentheses). The SPSS COUNT command consists of   *;
%*    variable lists and corresponding value lists. Each occurrence of        *;
%*    a match between one of the variables in a list and one of the values    *;
%*    in the corresponding list is counted and stored in a counter variable.  *;
%*    The macro call %_Count_ corresponds to the SPSS command COUNT, while    *;
%*    the first argument corresponds to the rest of the command, containing:  *;
%*  - a counter variable: counting the match between a variable and a value   *;
%*                     followed by the = character (like in SPSS)             *;
%*  - a variable list: individual variables, common {or SAS name literals},   *;
%*                     {array elements}, variable lists with _TO_ (or --),    *;
%*                     {constant numerical or character values (no _TO_)}     *;
%*                     [expressions with +, - and * (and / if NewCount Ne /)  *;
%*                      without spaces (unless Space is redefined).           *;
%*                      Expressions may work but are not intended/supported.] *;
%*  - a value list between parentheses, consisting of the following entries:  *;
%*                     individual valid values, individual missing values,    *;
%*                     all 28 numerical missing values or space (_SYSMIS_),   *;
%*                     value lists with _THRU_, {individual variables or      *;
%*                     array elements of which the value applies (no _TO_)}   *;
%*                     [expressions with +, - and * (and / if NewCount Ne /)  *;
%*                      without spaces (unless Space is redefined).           *;
%*                      Expressions may work but are not intended/supported.] *;
%*    optionally followed by more variable and value lists for the same       *;
%*    counter variable and so on: [varlist (vallist) [...(...)]] .            *;
%*    To start another counter variable, without ending the macro a slash (/) *;
%*    should be specified, followed by a new Counter variable and so on.      *;
%* 2. Support for SPSS-like _TO_ convention for implied variable lists (--)   *;
%* 3. Support of SAS quoted constant character values with any characters     *;
%* 4. Support of the SPSS-like _THRU_ convention for values. These define     *;
%*    numeric or character (lexicographical) value limits defining ranges     *;
%* 5. Missing value support with the _SYSMIS_ keyword: representing all 28    *;
%*    numeric (._ . .a to z) or one character (space) MISSING values          *;
%* 6. Support of the SPSS-like keywords _LO_ and _HI_ with a _THRU_ range,    *;
%*    (_LO_ by itself excluding missing values)                               *;
%* 7. Unsupported feature: limited specification of arithmetic expressions.   *;
%*                                                                            *;
%* In the summary above the extensions to the SPSS COUNT equivalent are       *;
%* enclosed by braces and brackets.                                           *;
%*                                                                            *;
%* Syntax:                                                                    *;
%* =======                                                                    *;
/* %_COUNT_ (( \* note: _two_ left parentheses / start of SPSS-like syntax *\ */
/*    Counter1 = var_list_1 (value_list_1) var_list_2 (value_list_2) [...] /  */
/*    Counter2 = var_list_3 (value_list_3) var_list_4 (value_list_4) [...] /  */
/*    [...].                                                                  */
/* \* end of SPSS-like syntax / note: _two_ consecutive right parentheses: *\ */
/*  ),\* optional keyword arguments here *\);                                 */
%*                                                                            *;
%* Output:                                                                    *;
%* =======                                                                    *;
%* One or more counting variables with counts of occurring specified values.  *;
%*                                                                            *;
%* Examples:                                                                  *;
%* =========                                                                  *;
/* %_COUNT_ (( \* note: _two_ left parentheses / start of SPSS-like syntax *\ */
/*    Cntvar1 = var_a (val_1, [..]) var_b (val_2 _THRU_ val_5, [..]) [..]     */
/*  / Cntvar2 = var_c var_d _TO_ var_f (value_6 value_7, [..]) [..] / [..].   */
/* \* end of SPSS-like syntax / note: _two_ consecutive right parentheses: *\ */
/*  ));                                                                       */
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* References:                                                                *;
%* ===========                                                                *;
%*                                                                            *;
%* 1. SAS macro %Count, Jim Groeneveld, PhUSE 2008, Manchester (coming up)    *;
%* 2. SAS macro validation criteria, Jim Groeneveld, PhUSE 2006, Dublin       *;
%*    http://home.hccnet.nl/jim.groeneveld/Recent/Summaries.shtml             *;
%* 3. http://tinyurl.com/parmbuff-at-comp-soft-sys-sas                        *;
%* or http://tinyurl.com/parmbuff-at-SAS-L                                    *;
%*                                                                            *;
%* Used auxiliary macros:                                                     *;
%* ----------------------                                                     *;
%* = internal: -                                                              *;
%* = attached: - DisClose, _ArrayNr                                           *;
%* = external: -                                                              *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of _Count_:   (initially developed named Count)                    *;
%* ===================                                                        *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.1      30 Nov 2005  first working release (with simple call syntax):     *;
%*                       %MACRO Count ( CountVar=, VarList=, ValList=)        *;
%*                          version released via website as COUNT.ZIP         *;
%* 0.2      16 Oct 2006  changed into processing unnamed arguments with       *;
%*                       PARMBUFF: %MACRO Count / PARMBUFF, callable as:      *;
%*                       %COUNT (Counter, a b c, 3 5 7 _sysmis_, b a, 1 9)    *;
%*                       [ on itself a rather good idea to develop further ]  *;
%*                       added auxiliary macro DisClose to remove parentheses *;
%* 0.2.1    17 Oct 2006  added keyword arg Debug, improved use of %DisClose:  *;
%*                       %COUNT ((Counter=a b c, (3 5 7 _sysmis_), (b a), 1 9));
%* 0.3      17 Oct 2006  removed unnamed arguments with PARMBUFF, keywords:   *;
%*                       %MACRO Count (Counter=, Counting=, Debug=0), called: *;
%*                       %COUNT (Counter=Counter, Counting = (a b c, (3 5 7   *;
%*                                                   _sysmis_), (b a), 1 9))  *;
%*                       Auxiliary macro %DisClose processes empty string     *;
%* 0.3.1    17 Oct 2006  changed %QUOTE into %BQUOTE, in %DisClose as well,   *;
%*                       Counting argument now positional, macro header:      *;
%*                       %MACRO Count (Counter=, Counting, Debug=0)           *;
%* 0.3.2    17 Oct 2006  removed Counter keyword, now in Counting, called as: *;
%*                       %COUNT ((Counter=a b c (3, 5, 7, _sysmis_)           *;
%*                                        b a (1, 9)), Debug=0)               *;
%* 0.3.3    17 Oct 2006  more (B)QUOTing and support for variables as values: *;
%*                       %COUNT ((Counter=a, b c (3, 5 7, _sysmis_)           *;
%*                                        b, a (1, 9) d (ParenL)              *;
%*                                        e f (BracketL BraceR)), Debug=0)    *;
%* 0.3.4    18 Oct 2006  minor changes, removed some calls to %DisClose       *;
%* 0.3.5    11 Aug 2008  changed <LE .z> into MISSING(var): also char.vars,   *;
%*                       more detailed tests with character values            *;
%* 0.3.6    11 Aug 2008  added support for _THRU_ SPSS-like keyword, testing  *;
%* 0.3.7    13 Aug 2008  changed arg name Counting into SPSSlike, support for *;
%*                       optional trailing SPSS-like dot ending the command,  *;
%*                       support for subcommands, subcounts separated by /    *;
%*                       Improved %Disclose to remove redundant spaces,       *;
%*                       Testing array specifications, extensive testing      *;
%* 0.3.8    14 Aug 2008  added _LO_ and _HI_ support with extensive testing   *;
%* 0.3.8a   14 Aug 2008  added notification if no value after _THRU_          *;
%* 0.3.9    15 Aug 2008  support for _TO_ convention for variable lists using *;
%*                       arrays, added aux macro %_ArrayNr, extensive testing *;
%* 0.3.9a   15 Aug 2008  minor changes to disallow array elements in varlists *;
%* 0.4      16 Aug 2008  changed (most) %[Q]SCAN into %QSYSFUNC(SCANQ to skip *;
%*                       scanning inside quoted strings. This enables to:     *;
%*                       - specify character values with embedded spaces and  *;
%*                         other special characters, like parentheses,        *;
%*                       - specify, support SAS name literals as variables,   *;
%*                       forced unquoted, generated SAS code                  *;
%* 0.4.1    16 Aug 2008  minor improvements (DROP temp index in SAS), testing *;
%* 0.4.2    18 Aug 2008  minor changes, removed debugging code, added comment *;
%* 0.9      18 Aug 2008  minor change (sum assignment into sum statement)     *;
%*                       final complete and working version "9.0" for paper   *;
%*                          version released via website as _COUNT_.ZIP       *;
%* 0.9.1    18 Aug 2008  no substantial algorithm changes, just header and    *;
%*                          documentation addition, aux. keyword arguments    *;
%* 0.9.2    07 Sep 2008  added macro variables with default keywords:         *;
%*                          ( GlobIncr, ArPrefix, IndexVar )                  *;
%* 0.9.3    09 Sep 2008  and separator characters for all possible terms:     *;
%*                          ( Assign, NewCount, LeftPar, RightPar, Space,     *;
%*                            Comma, Sysmis, To, Thru, Low, High, Period )    *;
%* 0.9.4    16 Sep 2008  completed history documentation, made to-do-list     *;
%*                          for near 0.9.x versions, mainly checks (below)    *;
%*                          version released via website as _COUNT_.ZIP       *;
%* 0.9.5    18 Nov 2008  added keyword _TILL_ for range excluding limits      *;
%* 0.9.y    in due time  ...additions of user specified parameters checks...  *;
%* 0.9.z    near future  ...final adaptations ... and possibly validation...  *;
%* 1.0.0    f_u_t_u_r_e  official production release of latest 0.9.z version  *;
%*                                                                            *;
%* Future of _Count_: No serious changes or additions to the main algorithm   *;
%* ================== as the complete functionality is ready and supported.   *;
%* + possible bug fixes (no bugs known currently)                             *;
%* + checking user input (macro call) more and more extensive if possible:    *;
%* - as much as possible checks on variable existence using VARNUM()   ???    *;
%* - check filled and length of rarely specified arguments, mutually unequal  *;
%* - check LeftPar and RightPar redefined simultaneously                      *;
%* + possibly more extensive documentation / comment within the program code  *;
%* + possibly intended and extensive support for expressions (arith+logical)  *;
%* + as far as possible meet the Guidelines for Standard SAS Macros           *;
%* + validation                                                               *;
%*                                                                            *;
%******************************************************************************;

 %LOCAL    /* Define internal macro variables explicitely local */
  MacName  /* This macros external name: _Count_ */
  MacVs    /* Version number major.minor.patchSub of macro: 0.9.5 */
  MacDate  /* Date of this version of macro: 18 Nov 2008 */
  ErrCount /* number of erroneously specified arguments */
  Err      /* text 'E R R O R' as consecutive characters */
  Warn     /* text 'W A R N I N G' as consecutive characters */
  ;

  %LET MacName  = _Count_;
  %LET MacVs    = 0.9.5;
  %LET MacDate  = 18 Nov 2008;
  %LET Err      = %SYSFUNC(COMPRESS(E R R O R, %STR( )));
  %LET Warn     = %SYSFUNC(COMPRESS(W A R N I N G, %STR( )));

  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN
  %DO;
    %DisClose (ForMyUse=Compile);
    %_ArrayNr (ForMyUse=Compile);
    %GOTO Finish;
  %END;

  %IF (&Debug EQ 1 OR &Debug GT 11) %THEN %DO;
    OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Macro header in LOG file;
%*_________________________;

  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT |   Macro &MacName, vs. &MacVs, by Jim Groeneveld, &MacDate   |;
  %PUT |     counts occurring value lists for (dataset) variables     |;
  %PUT |______________________________________________________________|;
  %PUT;

%* Feedback of specified arguments;
  %PUT &MacName: Review of argument specifications (incl. defaults);
  %*PUT &MacName: SPSScode = &SPSScode;
  %PUT &MacName: GlobIncr = &GlobIncr;
  %PUT &MacName: ArPrefix = &ArPrefix;
  %PUT &MacName: IndexVar = &IndexVar;
  %PUT &MacName: Debug    = &Debug;
  %PUT &MacName: Assign   = &Assign;
  %PUT &MacName: NewCount = &NewCount;
  %PUT &MacName: LeftPar  = &LeftPar;
  %PUT &MacName: RightPar = &RightPar;
  %PUT &MacName: Space    = &Space;
  %PUT &MacName: Comma    = &Comma;
  %PUT &MacName: Sysmis   = &Sysmis;
  %PUT &MacName: To       = &To;
  %PUT &MacName: Thru     = &Thru;
  %PUT &MacName: Till     = &Till;
  %PUT &MacName: Low      = &Low;
  %PUT &MacName: High     = &High;
  %PUT &MacName: Period   = &Period;

 %LOCAL    /* Define internal macro variables explicitely local */
  H I J    /* loop counters, index variables */
  SubCount /* sub specification counting with one count variable */
  Counter  /* contains count variable name for each outer loop */
  SpecList /* remaining SPSS (long) command without Counter code */
  VarList  /* variable list from SpecList, there may be more */
  ValList  /* value    list from SpecList, there may be more */
  Var      /* single variable name from VarList */
  ToVar    /* next variable name after Var from VarList */
  StopVar  /* subsequent variable name after ToVar VarList */
  Val      /* single value from ValList */
  ThruVal  /* next value after Val from ValList */
  Highval  /* subsequent value after ThruVal from ValList */
  Array    /* (incrementing) array number */
  Lower    /* either LE or LT */
  Greater  /* either GE or GT */
  ;

%* Upcase;
  %LET Sysmis = %QUPCASE ( &Sysmis );
  %LET To     = %QUPCASE ( &To );
  %LET Thru   = %QUPCASE ( &Thru );
  %LET Till   = %QUPCASE ( &Till );
  %LET Low    = %QUPCASE ( &Low );
  %LET High   = %QUPCASE ( &High );

%* remove enclosing parens;
  %LET SPSScode = %Disclose (Enclosed=&SPSScode, MaxDepth=9);
  %IF (%QSYSFUNC(SCANQ(&SPSScode,-1,&Period)) EQ) %THEN %LET SPSScode =
      %BQUOTE( %SUBSTR(&SPSScode, 1, %LENGTH(&SPSScode)-1) );

%* Separate count groups;
  %LET H = 1;
  %LET SubCount = %QSYSFUNC ( SCANQ ( &SPSScode, &H, &NewCount ) );
  %DO %WHILE (%BQUOTE(&SubCount) NE);
%* Extract counter var name (and remove its leading and trailing blanks);
    %LET Counter = %QSYSFUNC ( SCANQ ( &SubCount, 1, &Assign ) );
    %LET Counter = %QSYSFUNC ( COMPRESS ( &Counter, &Comma.&Space ) );
    &Counter = 0; * if not implicitly (re)set by sum statement;
/*  %LET SpecList = %QSUBSTR(&SubCount,%EVAL(%INDEX(&SubCount,&Assign)+1)); */
    %LET SpecList = %QSYSFUNC ( SCANQ ( &SubCount, 2, &Assign ) );  %* no more = !;
%* No more = characters expected outside constant string values, if more: error;
    %IF (%QSYSFUNC ( SCANQ ( &SubCount, 3, &Assign ) ) NE ) %THEN
      %PUT &MacName - ERROR, SPSScode contains  more than one &Assign character in group;
%* Extract elements and remove leading and trailing blanks in them;
    %LET VarList = %QSYSFUNC ( SCANQ ( &SpecList, 1, &LeftPar ) );
    %LET ValList = %QSYSFUNC ( SCANQ ( &SpecList, 2, &LeftPar.&RightPar ) );
    %IF (&ValList EQ) %THEN %DO;
      %PUT &MacName - WARNING, Empty value list with VarList=&VarList;
    %END;
    %ELSE %DO;
      %LET I = 1;
      %LET Var = %QSYSFUNC ( SCANQ ( &VarList, &I, &Comma.&Space ) );
      %DO %WHILE (%BQUOTE(&Var) NE);

%* Support for _TO_ convention using arrays (with incrementally numbered names);
        %LET ToVar = %QUPCASE(%QSYSFUNC(SCANQ(&VarList, &I+1, &Comma.&Space )));
        %LET StopVar =        %QSYSFUNC(SCANQ(&VarList, &I+2, &Comma.&Space ));
        %IF ((%BQUOTE(&ToVar) EQ &To OR %BQUOTE(&ToVar) EQ %STR(--))
          AND %BQUOTE(&StopVar) NE               /* _TO_ convention, var list */
          AND %SYSFUNC(SCANQ(&Var, 2, {[%STR(%())) EQ /* Var no array element */
          AND %SYSFUNC(SCANQ(&StopVar, 2, {[%STR(%())) EQ  /* StopVar neither */
          ) %THEN %DO;
          %LET Array = %_ArrayNr (_Counter=&GlobIncr); %* auxiliary macro;
             DROP &IndexVar; %* drop temporary and macro defined index variable;
%* create unique incremental array name, no valid order check of variables;
          %UNQUOTE ( ARRAY &ArPrefix&Array &Var -- &StopVar; ) %* unique array;
%* variable list itself and its amount is unknown to macro;
          %UNQUOTE ( DO &IndexVar = 1 TO DIM(&ArPrefix&Array); ) %* start of SAS loop;
          %LET Var = &ArPrefix&Array[&IndexVar]; %* redefine Var as array elements;
          %LET I = %EVAL ( &I + 2 ) ;   %* finally skip over _TO_ and StopVar;
        %END;
        %ELSE %DO; %* single variable;
          DO; %* nothing special, just to start a DO-END block with one iteration;
        %END;

        %LET J = 1;
        %LET Val = %QSYSFUNC ( SCANQ ( &ValList, &J  , &Comma.&Space ) );
        %DO %WHILE (%BQUOTE(&Val) NE);
          %LET ThruVal = %QUPCASE(%QSYSFUNC(SCANQ(&ValList, &J+1, &Comma.&Space )));
          %LET HighVal =          %QSYSFUNC(SCANQ(&ValList, &J+2, &Comma.&Space ));
          %IF (%BQUOTE(&ThruVal) NE &Thru AND %BQUOTE(&ThruVal) NE &Till) %THEN %DO;
            %IF (%QUPCASE(&Val) EQ &Sysmis) %THEN %DO;
              IF (MISSING(&Var)) THEN &Counter + 1;
* (all 28 SAS numerical missing values or character space are checked here);
            %END;
            %ELSE %DO;
              %UNQUOTE ( IF (&Var EQ &Val) THEN DO;
                &Counter + 1; )
              END;
%* can be simplified to: &Counter + (&Var EQ &Val);
            %END;
          %END;
%* _THRU_ support;
          %ELSE %DO; %* keyword is _THRU_ or _TILL_;
%* no _SYSMIS_ value expected here!;
          %IF (%BQUOTE(&ThruVal) EQ &Thru) %THEN %DO;
            %LET Lower = LE;
            %LET Greater = GE;
          %END;
          %ELSE %DO; %* keyword is _Till_;
            %LET Lower = LT;
            %LET Greater = GT;
          %END;
%* _LO_, _HI_ support;
            %IF (%BQUOTE(&HighVal) NE) %THEN %DO;
              IF ( %IF (%QUPCASE(&Val) EQ &Low) %THEN %DO;
                                NOT MISSING(&Var) AND    %END;
                   %ELSE %DO;   %UNQUOTE(&Var &Greater &Val AND)   %END;
                   %IF (%QUPCASE(&HighVal) EQ &High) %THEN %DO; 1 %END;
                   %ELSE %DO;   %UNQUOTE(&Var &Lower &HighVal)   %END;
                 ) THEN &Counter + 1;
%* can be simplified to: &Counter + ((interpreted) condition);
            %END;
            %ELSE %PUT _COUNT_ NOTE: Value lacking after keyword &Thru/&Till, no count!;
            %LET J = %EVAL (&J + 2); %* Proceed to after range spec;
          %END;
          %LET J = %EVAL (&J + 1);
          %LET Val = %QSYSFUNC ( SCANQ ( &ValList, &J, &Comma.&Space ) );
        %END;
          END; %* of loop through var list from _TO_ convention or normal;

        %LET I = %EVAL (&I + 1);
        %LET Var = %QSYSFUNC ( SCANQ ( &VarList, &I, &Comma.&Space ) );
      %END;
    %END;
    %LET H = %EVAL (&H + 1);
    %LET SubCount = %QSYSFUNC ( SCANQ ( &SPSScode, &H, &NewCount ) );
  %END;
 %Finish:
%MEND _Count_;

%* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * ;
%* E n d   o f   _ C o u n t _   (syntax conversion from simple to complex)    ;
%* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * ;

*~~~~~~~~~~~~~~~~~~;
* Auxiliary macros ;
*__________________;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Auxiliary Macro Disclose removes enclosing parentheses ;
%*________________________________________________________;

%MACRO Disclose (Enclosed=, MaxDepth=1, ForMyUse=);
  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN %GOTO Finish;
%* Remove leading and trailing spaces beforehand;
  %LET Enclosed = %BQUOTE(%UNQUOTE(&Enclosed));
  %IF (&Enclosed EQ %STR(%(%))) %THEN; %* if () then return nothing or msg;
                                               %* to prevent warning message;
  %ELSE %IF (%QSUBSTR(&Enclosed,1,1) EQ %STR(%()
         AND %QSUBSTR(&Enclosed,%LENGTH(&Enclosed),1) EQ %STR(%))) %THEN %DO;
    %IF (&MaxDepth LE 1) %THEN
/* truncate surrounding parentheses and remove leading and trailing spaces */
      %BQUOTE( %SUBSTR(&Enclosed, 2, %LENGTH(&Enclosed)-2) );
    %ELSE /* recursively */
      %Disclose (Enclosed=%QSUBSTR(&Enclosed, 2, %LENGTH(&Enclosed)-2),
                 MaxDepth=%EVAL(&MaxDepth-1));
  %END;
  %ELSE &Enclosed; %* return disclosed or original value;
 %Finish:
%MEND Disclose;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Auxiliary macro _ArrayNr creates or increments global macro variable _ArrayNr ;
%*_______________________________________________________________________________;

%MACRO _ArrayNr (_Counter=_ArrayNr, ForMyUse=);
  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN %GOTO Finish;
  %IF (%NRBQUOTE(&&&_Counter) EQ %NRSTR(&)&_Counter) %THEN %DO; %* Not existing;
    %* [ %SUPERQ(&_Counter) then is empty, but that is not enough ] ;
    %* WARNING: Apparent symbolic reference &_Counter not resolved. ;
    %PUT NOTE: Ignore WARNING about unresolved symbolic reference &_Counter above;
    %* Create GLOBAL macro variable &_Counter and initialize to 1 ;
    %GLOBAL &_Counter ; %LET &_Counter = 1;
  %END;
  %ELSE %DO; %* Existing;
    %* Increment &_Counter;
    %LET &_Counter = %EVAL ( &&&_Counter + 1 ) ;
  %END;
  &&&_Counter /* return value */
 %Finish:
%MEND;
